/** Example of solving BVP: x'(0)=x'(2pi)=0 **/
#include <iostream>
#include "capd/capdlib.h"
using namespace capd;
using namespace std;
int main(){
  IMap f("par:a;time:t;var:x,dx;fun:dx,-x*(1+x^2)/10 + a*cos(t);");
  f.setParameter("a",interval(4464)/interval(10000));
  IOdeSolver solver(f,20);  // ODE integrator
  ITimeMap tm(solver);      // class for long time integration

  IVector u0({-0.5072,0.});
  IVector r({interval(-1e-5,1e-5),0.});
  C0HOTripletonSet s0(u0); 
  C1HORect2Set s(u0+r); 
  // integrate until T=2*pi 
  IVector y = tm(2.*interval::pi(),s0);   
  tm(2.*interval::pi(),s);
  // solve equation F(r1) := proj_{x'}(phi(2pi,u0+(r1,0))) = 0
  interval N = - y[1]/((IMatrix)s)(2,1);
  cout << "(N,r1)=(" << N << "," << r[0] << ")"<< endl;
  cout << "subset(N,r)? = " << boolalpha << subset(N,r[0]) << endl;
  return 0;
}
/* Output:
(N,r1)=([-3.84493e-05, -2.09823e-05],[-0.1, 0.1])
subset(N,r)? = true
*/ 
