// Note: all examples in this tutorial require C++11 support
#include <iostream>
#include "capd/capdlib.h"
using namespace capd;
using namespace std;
int main(){
  IMap f("par:a;var:x,y;fun:sin(x*cos(y)),y*cos(a*y)-x^2;",3);
  f.setParameter("a",interval(1.,1.01));
  IVector u({1,3});
  cout << "f(u)=" << f(u) << endl;
  cout << "Df(u)=" << f.derivative(u) << endl;
  IJet jet(2,3);  // third order Taylor coefficients, two variables
  f(u,jet);       // compute full Taylor expansion
  cout << jet.toString() << endl;

  IOdeSolver solver(f,20);  // ODE integrator
  ITimeMap tm(solver);      // class for long time integration
  C0HOTripletonSet set(u);  // representation of initial condition
  // integrate until T=2 and print result
  cout << "phi(2,(1,3)) = " << tm(2.,set) << endl;   

  ITimeMap::SolutionCurve solution(0.);
  set = C0HOTripletonSet(IVector({1,1})); 
  // integrate until T=2 and save entire trajectory
  tm(2.,set,solution);
  cout << "solution(2)=" << solution(2) << endl;
  cout << "solution(0.9,1.1)=" << solution(interval(.9,1.1)) << endl;
}
