/** Example of integration of variational equations **/
#include <iostream>
#include "capd/capdlib.h"
using namespace capd;
using namespace std;
int main(){
  IMap lorenz("var:x,y,z;fun:10*(y-x),x*(28-z)-y,x*y-8*z/3;");
  IOdeSolver solver(lorenz,20);  // ODE integrator
  ITimeMap tm(solver);           // class for long time integration

  IVector u({1,5,23});
  // representation of initial condition, 
  // initial condition for variational equations is Id by default
  C1HORect2Set set(u); 
  // integrate until T=2 and print result 
  cout << "phi(2,u)=" << tm(2.,set) << endl;   
  // print monodromy matrix
  cout << "D_x phi(t,u) = " << (IMatrix)set << endl;
  
  ITimeMap::SolutionCurve solution(0.);
  C1HORect2Set s(u);
  // integrate and record trajectory 
  tm(2.,s,solution);
  cout << "solution(1) = " << solution(1) << endl;
  cout << "D_x solution(1.5) = " << solution.derivative(1.5) << endl;
  return 0;
}
/* Output of the program:
phi(2,u)={[3.57516, 3.57516],[-0.797643, -0.797643],[27.9102, 27.9102]}
D_x phi(t,u) = {
{[-1.95841, -1.95841],[-3.28406, -3.28406],[-0.614346, -0.614346]},
{[1.17314, 1.17314],[1.71972, 1.71972],[0.813136, 0.813136]},
{[-4.75217, -4.75217],[-7.68436, -7.68436],[-2.00251, -2.00251]}
}
solution(1) = {[11.8084, 11.8084],[16.4144, 16.4144],[25.3, 25.3]}
D_x solution(1.5) = {
{[0.974059, 0.974059],[1.87338, 1.87338],[-0.125996, -0.125996]},
{[-0.276187, -0.276187],[-0.249576, -0.249576],[-0.470697, -0.470697]},
{[3.59364, 3.59364],[6.48145, 6.48145],[0.30862, 0.30862]}
}*/

