/** Example of computation of derivative of Poincare map **/
#include <iostream>
#include "capd/capdlib.h"
using namespace capd;
using namespace std;
int main(){
  cout.precision(5);
  IMap lorenz("var:x,y,z;fun:10*(y-x),x*(28-z)-y,x*y-8*z/3;");
  IOdeSolver solver(lorenz,20);        // ODE integrator
  ICoordinateSection section(3,2,27.); // section is z=27
  IPoincareMap pm(solver,section);

  C1HORect2Set set(IVector({1,5,27}));
  IMatrix Dphi(3,3);
  IVector P = pm(set,Dphi);
  // recompute derivative of flow to derivative of Poincare map 
  IMatrix DP = pm.computeDP(P,Dphi);
  cout << "P(1,5,27)=" << P << endl;
  cout << "DP(1,5,27)=" << DP << endl;
}
/* Output:
P(1,5,27)={[11.361, 11.361],[14.338, 14.338],[27, 27]}
DP(1,5,27)={
{[-0.19415, -0.19415],[-0.46759, -0.46759],[-0.087997, -0.087997]},
{[-0.4121, -0.4121],[-0.96593, -0.96593],[-0.18836, -0.18836]},
{[-1.37e-13, 1.3706e-13],[-5.2713e-13, 5.2758e-13],[-2.5335e-13, 2.5335e-13]}
}
*/
