#include <iostream>
#include "capd/capdlib.h"
using namespace capd;
int main(){
  // f is an unperturbed vector field
  IMap f("var:x,y;fun:y,(1-x^2)*y-x;");
  // we define a perturbation e(t) \in [-eps,eps]
  IMap perturb("par:e;var:x,y;fun:e,e;");
  perturb.setParameter("e", interval(-1e-4, 1e-4));
  // We set right hand side of differential inclusion to f + perturb
  IMultiMap rhs(f, perturb);
  //  We set up two differential inclusions (order 20)
  // (they differ in the way they handle perturbations)
  CWDiffInclSolver cwSolver(rhs, 20, IMaxNorm());
  LNDiffInclSolver lnSolver(rhs, 20, IEuclLNorm());
  // constant time step, just for this example (not recommended)
  cwSolver.setStep(1./128); lnSolver.setStep(1./128);
  // Representation of initial condition for diff. incl. 
  InclRect2Set lnSet({2.,0.}), cwSet({2.0, 0.0});
  // We perform some numnber of steps with constant time step
  for(int i = 0; i < 128; ++i) {
    lnSet.move(lnSolver);
    cwSet.move(cwSolver);
  }
  std::cout.precision(10);
  std::cout << "LN method:\n" << IVector(lnSet) << std::endl;
  std::cout << "CW method:\n" << IVector(cwSet) << std::endl;
}
/* Output:
LN method:
{[1.507948164, 1.50834031],[-0.7803484048, -0.7800877445]}
CW method:
{[1.508005535, 1.508282938],[-0.7803100148, -0.7801261345]}
*/
