#include <iostream>
#include "capd/capdlib.h"
#include "capd/poincare/TimeMap.hpp" 
using namespace capd;
using namespace std;
typedef poincare::TimeMap<CWDiffInclSolver> CWTimeMap;
int main(){
  // f is an unperturbed vector field
  IMap f("var:x,y;fun:y,(1-x^2)*y-x;");
  // we define a perturbation e(t) \in [-eps,eps]
  IMap perturb("par:e;var:x,y;fun:e,e;");
  perturb.setParameter("e", interval(-1e-4, 1e-4));
  // We set right hand side of differential inclusion to f + perturb
  IMultiMap rhs(f, perturb);
  // component-wise based solver
  CWDiffInclSolver cwSolver(rhs, 20, IMaxNorm());
  // class for long-time integration with this solver
  CWTimeMap tm(cwSolver);
  // Representation of initial condition for diff. incl. 
  InclRect2Set set({2.,3.});
  cout.precision(13);
  cout << "phi(1,(2,3))=\n" << tm(1.,set);
}
/* Output:
phi(1,(2,3))=
{[2.300371385204, 2.300624075276],[-0.4798629375598, -0.4797786804589]}
*/
