/** Existence of invariant curves around ellipitic PO **/
#include <iostream>
#include "capd/capdlib.h"
#include "capd/normalForms/planarMaps.hpp"
using namespace capd;
int main(){
  IMap pendulum("time:t;var:x,dx;fun:dx,sin(t)-sin(x);",3);
  ICnOdeSolver solver(pendulum,20);
  ICnTimeMap tm(solver);           
  // validate existence of a periodic point by Newton method
  IVector u0({0,-2.24910979679593});
  IVector r = interval(-1e-12,1e-12)*IVector{1,1};
  C0TripletonSet s0(u0);
  IVector y =  tm(2*interval::pi(),s0);
  C1Rect2Set s(u0+r);
  tm(2*interval::pi(),s);
  IMatrix DP = IMatrix(s) - IMatrix::Identity(2);
  IVector N = - matrixAlgorithms::gauss(DP,y-u0);
  std::cout << "subset(N,r)? = " << subset(N,r) 
            << "\nN = " << N << "\nr = " << r << std::endl;
  //integrate 3rd order variational equations and compute normal form
  CnRect2Set S(u0+N,3);
  tm(2*interval::pi(),S); 
  std::cout << "gamma1 = " << 
    normalForms::computePlanarElipticNormalForm(S.currentSet())[1]
    << "\n(should have non-zero real part)";
  return 0;
}
/* Output:
subset(N,r)? = 1
N = {[-4.465e-14, 4.22254e-14],[-1.45119e-14, 1.41125e-14]}
r = {[-1e-12, 1e-12],[-1e-12, 1e-12]}
gamma1 = ([0.937576, 0.937576],[-1.0637e-08, 1.0637e-08])
(should have non-zero real part)
*/
