#include <iostream>
#include "capd/capdlib.h"
using namespace capd; using namespace std;
int main(){
  IMap lorenz("var:x,y,z;fun:10*(y-x),x*(28-z)-y,x*y-8*z/3;");
  IOdeSolver solver(lorenz,20);        // ODE integrator
  ICoordinateSection section(3,2,27.); // section is z=27
  IPoincareMap pm(solver,section);
  // very rough approximation of a periodic point
  IVector u0({-2.14737, 2.07805, 27});
  IVector r = IVector({1.,1.,0.})*interval(-1e-5,1e-5);
  C0HOTripletonSet s0(u0);
  // compute fu0:=P^4(u0)-u0 and project it onto (x,y)
  IVector fu0( 2, (pm(s0,4) - u0).begin() ); 
  // compute derivative 
  C1HORect2Set s1(u0+r);
  IMatrix Dphi(3,3);
  IVector Pu = pm(s1,Dphi,4);
  IMatrix DP = pm.computeDP(Pu,Dphi);
  // projection of DP^4(u0+r)-Id onto 2D subspace
  IMatrix M({{DP(1,1)-1,DP(1,2)},{DP(2,1),DP(2,2)-1}});
  // enclose -(DP^4(u0+r)-Id)^{-1}*(P^4(u0)-u0)
  IVector N = - matrixAlgorithms::gauss(M,fu0);
  cout << "validated? " << subset(N,IVector(2,r.begin())) << endl; 
  cout << "DP=" << IMatrix({{DP(1,1),DP(1,2)},{DP(2,1),DP(2,2)}});
  // explicit formula for eigenvalues of a 2x2 matrix
  interval t = sqrt(4*DP(2,1)*DP(1,2) + sqr(DP(1,1)-DP(2,2))); 
  cout << "\nlambda1=" << 0.5*(DP(1,1)+DP(2,2)-t);
  cout << "\nlambda2=" << 0.5*(DP(1,1)+DP(2,2)+t);
}
/* Output of the program:
validated? 1
DP={
{[1.01919, 1.0261],[2.72595, 2.74535]},
{[1.37796, 1.38107],[3.68603, 3.69458]}
}
lambda1=[-0.00979719, 0.00979936]
lambda2=[4.70315, 4.72275]
*/
