#include <iostream>
#include "capd/capdlib.h"
using namespace capd;

void validateSymPO(interval y, int iteration){
  IMap vf("var:x,y,z;fun:y,z,1-y-0.5*x^2;"); // vector field
  IOdeSolver solver(vf, 20);                 // 20th order solver    
  ICoordinateSection section(3, 2);          // section z=0
  IPoincareMap pm(solver, section);

  // Check that the Poincare map is defined on (0,y)
  C0HOTripletonSet s(IVector({0.,y,0.}));
  std::cout << "P(0,y)=" << pm(s,iteration) << std::endl;
  
  // Compute P(0,left(y)) and P(0,right(y))
  C0HOTripletonSet s1(IVector({0.,y.left(),0.}));
  C0HOTripletonSet s2(IVector({0.,y.right(),0.}));
  IVector r1 = pm(s1,iteration);
  IVector r2 = pm(s2,iteration);
  // Check that x-coordinate changes the sign
  std::cout << "check inclusion? " << ( r1[0]*r2[0]<0 ) << std::endl;
}
int main(){
  // Call the algorithm with two approximate periodic points
  validateSymPO(1.5259617305036892  + interval(-1,1)*1e-15, 1);
  validateSymPO(0.50002564853520548 + interval(-1,1)*1e-13, 2);
  return 0;
}
