#include <cstdlib>
#include <iostream>
#include <iomanip>
#include <cmath>
#include "capd/krak/krak.h"
#include "capd/capdlib.h"
using namespace std;
using namespace capd;

Frame fr, txt;


// uncomment the following line if you wish to wait for a key after each step 
#define _WAIT_FOR_KEY_

// Ranges for graphical window
double minx=-3.5, maxx=3.5, miny=-3.5,  maxy=3.5;

double eps   = 1.0e-1;                             ///<  maximal forcing
int   nSteps = 300;                                 ///<  number of steps in one period 
double timeStep  = interval::pi().rightBound()/nSteps; ///<  time step for integration
int    order = 10;                                 ///<  order of IOdeSolver method
double bSize = 0.01;

int readKey()
{
  int s=NO_KEY;
  UserMove um;
#ifdef _WAIT_FOR_KEY_
  while(NO_KEY==s) 
#endif
  {
    GetUserMove(um);
    s=um.key;
  }
  return s;
}

void waitForKey()
{
#ifndef _WAIT_FOR_KEY_
    rootFrame << " Hit any key ! ";
    waitBt();
#endif
}
// -----------------------------------------------------------------

void initGraphics()
{
   openGW(900,700,WHITE,BLACK);
   rootFrame.Clear();
   txt = Frame(0,0,595,130);
   fr = Frame(5,135,985,685,WHITE,BLACK);
   fr.setWorldCoord(minx,miny,maxx,maxy);
   fr.line(0.0,miny,0.0,maxy,BLACK);
   fr.line(minx,0.0,maxx,0.0,BLACK);
}

void drawRectangle(std::vector<IVector> & cor, Frame & fr, int color){
  
  for(int i = 0; i<4; i++){
    for(int j=0; j<4; j++){
      fr.line( cor[i][0].mid().leftBound(), cor[i][1].mid().leftBound(),cor[j][0].mid().leftBound(),cor[j][1].mid().leftBound(), color);
    }
    fr.boxFill(cor[i][0].leftBound(), cor[i][1].leftBound(),cor[i][0].rightBound(), cor[i][1].rightBound(),color,SOLID_P);
  }
    
}

void inclusionTest(){
  
  
 
  // We set right hand side of differential inclusion = f + perturb
  //  it is a harmonic oscilator with the perturbation of a size eps
  IMap f("var:x,y;fun:y,-x;");
  IMap perturb("par:e;var:x,y;fun:0,e;");
  perturb.setParameter("e",interval(-eps, eps));
  
  IMultiMap rhs(f, perturb);
 
  // We set up two differential inclusions (they differ in the way they handle perturbations)
  CWDiffInclSolver diffInclCW(rhs, 10,  capd::IMaxNorm());  
  LNDiffInclSolver diffInclLN(rhs, 10, capd::IEuclLNorm());
  diffInclCW.setStep(timeStep);
  diffInclLN.setStep(timeStep);
  IMap vf("time:t;par:e;var:x,y;fun:y,-x+e*sin(t);");
  vf.setParameter("e",interval(eps));

  IOdeSolver solver(vf, order);
  solver.setStep(timeStep);
  C1Rect2Set S({-1.,0.});
    
  // Starting point
  IVector x0({-1.,0.});
  InclRect2Set set1(x0), set2(x0) ;
  IVector v(2);
  // We do MAX number of steps
  int s=NO_KEY;
  SetFgCol(BLUE);
  std::vector<IVector> cor;
  do{
    fr.Clear();
    set2.move(diffInclLN);
    cor = set2.getCorners();
    drawRectangle(cor, fr, RED);
    
    /*
    v = IVector(set2);
    fr.boxFill(v[0].leftBound(), v[1].leftBound(),v[0].rightBound(), v[1].rightBound(),YELLOW,SOLID_P);
    fr.dot(v[0].leftBound(),v[1].leftBound());
    */
    
    
    set1.move(diffInclCW);    
    cor = set1.getCorners();
    drawRectangle(cor, fr, BLUE);
    
    
    /*
    v = IVector(set1);
    fr.boxFill(v[0].leftBound(), v[1].leftBound(),v[0].rightBound(), v[1].rightBound(),BLUE,SOLID_P);
    fr.dot(v[0].leftBound(),v[1].leftBound());
   */
     S.move(solver);
      v = IVector(S);
            fr.boxFill(v[0].leftBound()-bSize, v[1].leftBound()-bSize,v[0].rightBound()+bSize, v[1].rightBound()+bSize,BLACK);

      fr.dot(v[0].leftBound(),v[1].leftBound());
    
   s=readKey(); 
    
  }while(s!=EscKey);
  waitForKey();
  
  IVector result = IVector(set1);
  std::cout.precision(16);
  rootFrame<< "\n\n Method based on component wise estimates : \n " <<  result
            << "\n  diam = " << diam(result)
            << "\n";
  result = IVector(set2);
  rootFrame<< "\n\n Method based on logarithmic norms : \n " <<  result
            << "\n  diam = " << diam(result)
            << "\n";
            
}

void doublePendulum(){
  
  int MAX = 10000;
  double timeStep = 6.283 /MAX ;
  
  // We set right hand side of differential inclusion = f + perturb
  //  it is a harmonic oscilator with the perturbation of a size sigma
  IMap f("var:x,y,k,l;fun:-y,x,-5*l,5*k;");
  IMap perturb("par:e1,e2;var:x,y,k,l;fun:e1,e1,e2,e2;");
  double sigma = 0;
  perturb.setParameter("e1",interval(-sigma, sigma));
  perturb.setParameter("e2",interval(-sigma, sigma));
  IMultiMap rhs(f, perturb);
 
  // We set up two differential inclusions (they differ in the way they handle perturbations)
  CWDiffInclSolver diffInclCW(rhs, 10,  capd::IMaxNorm());  
  LNDiffInclSolver diffInclLN(rhs, 10, capd::IEuclLNorm());
  diffInclCW.setStep(timeStep);
  diffInclLN.setStep(timeStep);
  
  // Starting point
  IVector x0(4);
  x0[0] = 10; x0[1] = 0;
  x0[2] = 10; x0[3] = 0;
  InclRect2Set set1(x0), set2(x0) ;
  
  int c = RED;
  int c2 = BLUE;
  // We do MAX number of steps
  for(int i=0; i<MAX; ++i){
    //rootFrame<< "  step " << i << " :";
    set1.move(diffInclCW);
    IVector w = IVector(set1);
    cout << w << endl;
    fr.boxFill(w[0].leftBound(),w[1].leftBound(),w[0].rightBound(),w[1].rightBound(),c);
    fr.box(w[0].leftBound(),w[1].leftBound(),w[0].rightBound(),w[1].rightBound());
    fr.boxFill(w[2].leftBound(),w[3].leftBound(),w[2].rightBound(),w[3].rightBound(),c);
    fr.box(w[2].leftBound(),w[3].leftBound(),w[2].rightBound(),w[3].rightBound());
    fr.dot(w[2].leftBound(),w[3].leftBound()); 
    set2.move(diffInclLN);
    w = IVector(set2);
    fr.boxFill(w[0].leftBound(),w[1].leftBound(),w[0].rightBound(),w[1].rightBound(),c2);
    fr.box(w[0].leftBound(),w[1].leftBound(),w[0].rightBound(),w[1].rightBound());
    fr.boxFill(w[2].leftBound(),w[3].leftBound(),w[2].rightBound(),w[3].rightBound(),c2);
    fr.box(w[2].leftBound(),w[3].leftBound(),w[2].rightBound(),w[3].rightBound());

    //rootFrame<< " *"<< "\n";
  }
  
  IVector result = IVector(set1);
  std::cout.precision(16);
  std::cout<< "\n\n Method based on component wise estimates : \n " <<  result
            << "\n  diam = " << diam(result)
            << "\n";
  result = IVector(set2);
  std::cout<< "\n\n Method based on logarithmic norms : \n " <<  result
            << "\n  diam = " << diam(result)
            << "\n";
            
}


int main( int, char*[])
{
   initGraphics();

  try{
 
    inclusionTest();
    //doublePendulum();
     
  } catch(capd::intervals::IntervalError<double> &e)
   {
      rootFrame <<" interval Error : " << e.what();
   }

   catch(std::runtime_error &e)
   {
      cout << e.what();
   }
   waitBt();
   closeGW();
   return 0;
}


