#include <iostream>
#include "capd/capdlib.h"
#include "capd/mpcapdlib.h"
using namespace capd; using namespace std;
void check(MpFloat y, MpFloat z, int n, double e=1e-22){
 MpIMap rossler("var:x,y,z;fun:-(y+z),x+0.2*y,0.2+z*(x-5.7);");
 MpIOdeSolver solver(rossler,50);      // ODE integrator
 MpICoordinateSection section(3,0.);   // section is x=0
 MpIPoincareMap pm(solver,section, poincare::MinusPlus);
 // approximate periodic point and a ball
 MpIVector u0({MpInterval(0.),y,z});
 MpIVector r({MpInterval(0.),MpInterval(-e,e),MpInterval(-e,e)});
 MpC0TripletonSet s0(u0);
 // compute P^n(u0)-u0 and project it onto (y,z)
 MpIVector fu0( 2, (pm(s0,n) - u0).begin() + 1 ); 
 MpC1Rect2Set s1(u0+r);   // compute derivative on u0+r
 MpIMatrix Dphi(3,3);
 MpIVector u = pm(s1,Dphi,n);
 MpIMatrix DP = pm.computeDP(u,Dphi);
 // projection of DP^n(u0+r)-Id onto 2D subspace
 MpIMatrix M({{DP(2,2)-1.,DP(2,3)},{DP(3,2),DP(3,3)-1.}});
 // enclose -(DP^n(u0+r)-Id)^{-1}*(P^n(u0)-u0)
 MpIVector N = - matrixAlgorithms::gauss(M,fu0);
 cout << boolalpha << "\n(validated?, accuracy) = " 
      << subset(N,MpIVector(2,r.begin()+1)) << ", " << maxWidth(N);
}
int main(){
 MpFloat::setDefaultPrecision(200);
 check("-8.38094174282987645183593","0.02959006063066710383300745",1);
 check("-5.42407382266520422640036","0.03108121080787644620332608",2);
 check("-6.23315862853797465596029","0.03064011165816057058379228",3);
}
/* Output of the program:
(validated?, accuracy) = true, 7.87471e-41 
(validated?, accuracy) = true, 3.35732e-41 
(validated?, accuracy) = true, 3.36216e-41 */
