#include <iostream>
#include "capd/capdlib.h"
using namespace capd;

int main(){
  IMap vf("var:x,y,z;fun:-(y+z),x+0.2*y,0.2+z*(x-5.7);");
  IOdeSolver solver(vf, 20);
  ICoordinateSection section(3, 0); // section x=0, x'>0
  IPoincareMap pm(solver, section, poincare::MinusPlus);

  // Coordinates of the trapping region
  const double B = 0.028, T = 0.034, L = -10.7, R = -2.3;

  // Subdivide uniformly onto 200 rectangles
  const int N = 160;
  bool result = true;
  interval p = (interval(R) - interval(L)) / N;
  for (int i = 0; i < N and result; ++i) {
    IVector x ({0., L + interval(i,i+1)*p, interval(B, T)});
    C0HOTripletonSet s(x);
    IVector u = pm(s);
    result = result and u[2]>B and u[2]<T and u[1]>L and u[1]<R;
    if(!result)
      std::cout << "Inclusion not satisfied:\n" << u << std::endl;
  }
  std::cout << "Existence of attractor: " << result << std::endl;
  return 0;
}
