#include <iostream>
#include "capd/capdlib.h"
using namespace capd;
using namespace std;

int main(){
  IMap vf("var:x,y,z;fun:-(y+z),x+0.2*y,0.2+z*(x-5.7);");
  IOdeSolver solver(vf, 20);
  ICoordinateSection section(3, 0); // section x=0, x'>0
  IPoincareMap pm(solver, section, poincare::MinusPlus);

  // z-coordinate of the trapping region
  interval z(0.028,0.034);
  // Coordinates of M and N
  const double lM=-8.4, rM=-7.6, lN=-5.7, rN=-4.6;

  C0HOTripletonSet LM( IVector({0.,lM,z}) ); 
  C0HOTripletonSet RM( IVector({0.,rM,z}) ); 
  C0HOTripletonSet LN( IVector({0.,lN,z}) ); 
  C0HOTripletonSet RN( IVector({0.,rN,z}) ); 

  // Inequalities for the covering relations N=>N, N=>M, M=>M, M=>N.
  cout << "P^2(LM) < lM: " << ( pm(LM,2)[1] < lM ) << endl;
  cout << "P^2(RM) > rN: " << ( pm(RM,2)[1] > rN ) << endl;
  cout << "P^2(LN) > rN: " << ( pm(LN,2)[1] > rN ) << endl;
  cout << "P^2(RN) < lM: " << ( pm(RN,2)[1] < lM ) << endl;  
  return 0;
}
