#include <iostream>
#include "capd/capdlib.h"
using namespace capd;
using namespace std;
bool checkCC(IPoincareMap& pm, double y1, double y2, int N) {
  bool res = true;
  interval p = (interval(y2) - interval(y1)) / N;
  IMatrix Dphi(3,3);
  IMatrix Q({{0.,0.,0.},{0.,1,0.},{0.,0.,-100}});
  interval returnTime;
  for (int i = 0; i < N and res; ++i) {
    C1Rect2Set s({0.,y1+interval(i,i+1)*p,interval(0.028,0.034)});
    IVector y = pm(s, Dphi, returnTime, 2);
    IMatrix DP = pm.computeDP(y,Dphi);
    DP = Transpose(DP)*Q*DP - Q;
    res = res and DP(2,2)>0 and (DP(2,2)*DP(3,3)-sqr(DP(2,3)))>0;
  }
  return res;
}
int main(){
  IMap vf("var:x,y,z;fun:-(y+z),x+0.2*y,0.2+z*(x-5.7);");
  IOdeSolver solver(vf, 20);
  ICoordinateSection section(3, 0); // section x=0, x'>0
  IPoincareMap pm(solver, section, poincare::MinusPlus);
  const double lM=-8.4, rM=-7.6, lN=-5.7, rN=-4.6;
  cout << "Cone condition on M: " << checkCC(pm,lM,rM,80) << endl;
  cout << "Cone condition on N: " << checkCC(pm,lN,rN,20) << endl;
}
