#include <iostream>
#include "capd/capdlib.h"
using namespace capd;
using namespace std;
int main(){
  // Instance of the vector field, ODE solver, 
  // Poincare section (y=0 is index=1 of 2 variables)
  // and the crossing direction (y changes sign from Plus to Minus)
  IMap vf("var:x,y;fun:y,(1-x^2)*y-x;");
  IOdeSolver solver(vf, 20);
  ICoordinateSection section(2, 1);
  IPoincareMap pm(solver, section, poincare::PlusMinus);
  
  // Take a ball centred at approximate periodic point
  IVector x({2.0086198608748433 + interval(-1,1)*1e-5,0.});  
  C0HOTripletonSet s(x);  
  // Call routine that computes rigorously Poincare map
  IVector y = pm(s);
  // check inclusion and print output
  cout.precision(17);
  cout << "y=" << y[0] << "\nx=" << x[0] << endl; 
  cout << boolalpha << "inclusion? = " << subset(y[0],x[0]);
  return 0;
} 
/* Output:
y=[2.0086198481018909, 2.0086198740315355]
x=[2.0086098608748433, 2.0086298608748434]
inclusion? = true */ 
